@@ -12,7 +12,9 @@ using MonoDevelop.Projects.Text;
 using MonoDevelop.Core;
 using MonoDevelop.Ide.Gui;
 
-using CSharpBinding;
+using MonoDevelop.Core.Assemblies;
+using MonoDevelop.CSharp.Project;
+
 
 namespace MonoDevelop.Prj2Make
 {
@@ -27,6 +29,8 @@ namespace MonoDevelop.Prj2Make
 		private bool m_bIsUnix;
 		private bool m_bIsMcs;
 		private bool m_bIsUsingLib;
+		
+		TargetFramework fx = Runtime.SystemAssemblyService.GetTargetFramework ("1.1");
  
 		// Flag use to determine if the LIB variable will be used in
 		// the Makefile that prj2make generates
@@ -166,14 +170,18 @@ namespace MonoDevelop.Prj2Make
 			}
 		}
     
-		public DotNetProject CreatePrjxFromCsproj (string csprojFileName, IProgressMonitor monitor)
+		public SolutionEntityItem CreatePrjxFromCsproj (string csprojFileName, IProgressMonitor monitor)
 		{
 			try {
+				FileFormat format = Services.ProjectService.FileFormats.GetFileFormatForFile (csprojFileName, typeof(SolutionEntityItem));
+				if (format != null && format.Id != "VS2003ProjectFileFormat")
+					return (SolutionEntityItem) Services.ProjectService.ReadSolutionItem (monitor, csprojFileName);
+				
 				MonoDevelop.Prj2Make.Schema.Csproj.VisualStudioProject csprojObj = null;
 				
 				monitor.BeginTask (GettextCatalog.GetString ("Importing project: ") + csprojFileName, 4);
 				
-				DotNetProject prjxObj = new DotNetProject ("C#", null, null);
+				DotNetAssemblyProject prjxObj = new DotNetAssemblyProject ("C#", null, null);
 				
 				prjxFileName = String.Format ("{0}.mdp",
 					Path.Combine (Path.GetDirectoryName (csprojFileName),
@@ -198,7 +206,7 @@ namespace MonoDevelop.Prj2Make
 				GetContents (prjxObj, csprojObj.CSHARP.Files.Include, prjxObj.Files, monitor);
 				monitor.Step (1);
 				
-				GetReferences (csprojObj.CSHARP.Build.References, prjxObj.References, monitor);
+				GetReferences (prjxObj, csprojObj.CSHARP.Build.References, prjxObj.References, monitor);
 				monitor.Step (1);
 				
 				prjxObj.Configurations.Clear ();
@@ -244,12 +252,21 @@ namespace MonoDevelop.Prj2Make
 						monitor.ReportWarning (GettextCatalog.GetString ("Project file not found: ") + pi.csprojpath);
 						continue;
 					}
-					DotNetProject prj = CreatePrjxFromCsproj (mappedPath, monitor);
+					SolutionEntityItem prj;
+					if (pi.NeedsConversion)
+						prj = CreatePrjxFromCsproj (mappedPath, monitor);
+					else
+						prj = (DotNetProject) Services.ProjectService.ReadSolutionItem (monitor, mappedPath);
+					
 					if (prj == null)
 						return null;
 
 					monitor.Step (1);
 					solution.RootFolder.Items.Add (prj);
+					foreach (ItemConfiguration conf in prj.Configurations) {
+						if (!solution.GetConfigurations ().Contains (conf.Id))
+							solution.AddConfiguration (conf.Id, false);
+					}
 					monitor.Step (1);
 				}
 				
@@ -272,7 +289,7 @@ namespace MonoDevelop.Prj2Make
 			}
 		}
 		
-		protected void GetReferences (MonoDevelop.Prj2Make.Schema.Csproj.Reference[] References, ProjectReferenceCollection references, IProgressMonitor monitor)
+		protected void GetReferences (Project project, MonoDevelop.Prj2Make.Schema.Csproj.Reference[] References, ProjectReferenceCollection references, IProgressMonitor monitor)
 		{
 			if (References == null || References.Length == 0)
 				return;
@@ -326,8 +343,16 @@ namespace MonoDevelop.Prj2Make
 						if (rname == "System.XML")
 							rname = "System.Xml";
 						
-						string oref = Runtime.SystemAssemblyService.GetAssemblyFullName (rname);
+						string oref = Runtime.SystemAssemblyService.DefaultAssemblyContext.GetAssemblyFullName (rname, fx);
 						if (oref == null) {
+							if (rf.HintPath != null) {
+								string asm = MapPath (project.ItemDirectory, rf.HintPath);
+								if (!System.IO.File.Exists (asm))
+									monitor.ReportWarning (GettextCatalog.GetString ("Referenced assembly not found: ") + asm);
+								ProjectReference aref = new ProjectReference (MonoDevelop.Projects.ReferenceType.Assembly, asm);
+								references.Add (aref);
+								continue;
+							}
 							monitor.ReportWarning (GettextCatalog.GetString ("Assembly reference could not be imported: ") + rf.AssemblyName);
 							continue;
 						}
@@ -384,7 +409,7 @@ namespace MonoDevelop.Prj2Make
 
 			// Test to see if file exist in GAC location
 			if (System.IO.File.Exists (strRefFileName)) {
-				ProjectReference rfOut = new ProjectReference (MonoDevelop.Projects.ReferenceType.Gac, Runtime.SystemAssemblyService.GetAssemblyFullName (strRefFileName));
+				ProjectReference rfOut = new ProjectReference (MonoDevelop.Projects.ReferenceType.Gac, Runtime.SystemAssemblyService.DefaultAssemblyContext.GetAssemblyFullName (strRefFileName, fx));
 				rfOut.LocalCopy = true;
 				return rfOut;
 			}
@@ -454,7 +479,7 @@ namespace MonoDevelop.Prj2Make
 			
 			string dir = MapPath (project.BaseDirectory, ConfigBlock.OutputPath);
 			if (dir == null) {
-				dir = "bin/" + ConfigBlock.Name;
+				dir = Path.Combine ("bin", ConfigBlock.Name);
 				monitor.ReportWarning (string.Format (GettextCatalog.GetString ("Output directory '{0}' can't be mapped to a local directory. The directory '{1}' will be used instead"), ConfigBlock.OutputPath, dir));
 			}
 			confObj.OutputDirectory = dir;
